#include "global.h"
#include <string>
#include <d3dx9.h>
#include "texturemanager.h"
#include "textureobject.h"
#include "meshobject.h"

void CMMMesh::CleanUp()
{
	SAFE_RELEASE( mesh );
	filename = L"";
	SAFE_DELETE_ARRAY( materials );
    if( textures )
    {
        for( DWORD i = 0; i < nummaterials; ++i )
        {
			SAFE_RELEASE( textures[i] );
		};
        delete[] textures;
    };
	nummaterials = 0;
	D3DXMatrixTranslation( &translation, 0.0f, 0.0f, 0.0f );
	D3DXMatrixRotationYawPitchRoll( &rotation, 0.0f, 0.0f, 0.0f );
	D3DXMatrixScaling( &scaling, 1.0f, 1.0f, 1.0f );
	SetupMatrix();
};

HRESULT CMMMesh::LoadMesh( LPDIRECT3DDEVICE9 DEVICE, LPWSTR FILENAME )
{
	LPD3DXBUFFER pD3DXMtrlBuffer;

	CleanUp();

	if( FAILED( D3DXLoadMeshFromX( FILENAME, D3DXMESH_SYSTEMMEM, 
									DEVICE, NULL, 
									&pD3DXMtrlBuffer, NULL, &nummaterials, 
									&mesh ) ) )
	{
		return E_FAIL;
	};

	D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
	materials = new D3DMATERIAL9[nummaterials];
	textures = new CMMTexture*[nummaterials];

	for( DWORD i = 0; i < nummaterials; ++i )
    {
		materials[i] = d3dxMaterials[i].MatD3D;
		materials[i].Ambient = materials[i].Diffuse;

		textures[i] = NULL;
		UINT fnLength = ( UINT )lstrlenA( d3dxMaterials[i].pTextureFilename );
		if( d3dxMaterials[i].pTextureFilename != NULL && 
			fnLength > 0 )
		{
			LPWSTR FileName = new WCHAR[fnLength + 1];
			mbstowcs( FileName, d3dxMaterials[i].pTextureFilename, fnLength + 1 );

			CMMTexture* TObj = 
				Textures::TextureManager->FindTexture( FileName );
			if( TObj )
			{
				TObj->AddRef();
			}
			else
			{
				TObj = Textures::TextureManager->CreateTextureObject( TEXTURE_SIMPLE );
				TObj->LoadTexture( DEVICE, FileName );
			};
			textures[i] = TObj;
		};
	}
	pD3DXMtrlBuffer->Release();

	D3DXVECTOR3* pData; 
	mesh->LockVertexBuffer( 0, (LPVOID*) &pData );
	D3DXComputeBoundingSphere( pData, mesh->GetNumVertices(),
		D3DXGetFVFVertexSize( mesh->GetFVF() ), &meshcenter, &meshradius );
	mesh->UnlockVertexBuffer();
	SetupMatrix();

	return S_OK;
};

void CMMMesh::DrawMesh( LPDIRECT3DDEVICE9 DEVICE )
{
	DEVICE->SetTransform( D3DTS_WORLD, &matrix );
	for( DWORD i = 0; i < nummaterials; ++i )
	{
		DEVICE->SetMaterial( &materials[i] );
		if( textures[i] )DEVICE->SetTexture( 0, textures[i]->GetTexture() );
		else DEVICE->SetTexture( 0, NULL );
		mesh->DrawSubset( i );
	};
};

LPD3DXMESH CMMMesh::GetMesh()
{
	return mesh;
};

D3DXVECTOR3 CMMMesh::GetMeshCenter()
{
	return meshcenter;
};

float CMMMesh::GetMeshRadius()
{
	return meshradius;
};

std::wstring CMMMesh::GetMeshFileName()
{
	return filename;
};

D3DMATRIX CMMMesh::GetMatrix()
{
	return matrix;
};

void CMMMesh::SetTranslation( D3DMATRIX TRANSLATION )
{
	translation = D3DXMATRIXA16(TRANSLATION);
	SetupMatrix();
};

D3DMATRIX CMMMesh::GetTranslation()
{
	return translation;
};

void CMMMesh::SetRotation( D3DMATRIX ROTATION )
{
	rotation = D3DXMATRIXA16(ROTATION);
	SetupMatrix();
};

D3DMATRIX CMMMesh::GetRotation()
{
	return rotation;
};

void CMMMesh::SetScaling( D3DMATRIX SCALING )
{
	scaling = D3DXMATRIXA16(SCALING);
	SetupMatrix();
};

D3DMATRIX CMMMesh::GetScaling()
{
	return scaling;
};

void CMMMesh::SetupMatrix()
{
	D3DXMatrixTranslation( &matrix, -meshcenter.x, -meshcenter.y, -meshcenter.z );
	matrix *= rotation;
	matrix *= scaling;
	matrix *= translation;
};