#include "global.h"
#include "particlesystem.h"
#include "particleobject.h"

void CMMParticleSystem::CleanUp()
{
	for( int i = 0; i < particlenum; ++i )
	{
		SAFE_DELETE( particles[i] );
	};
	SAFE_DELETE_ARRAY( particles );
	SAFE_RELEASE( texture );
	SAFE_RELEASE( vertex );
	SAFE_DELETE_ARRAY( buffer );
	particlenum = 0;
	particlesize = 0.0f;
	initparticle = NULL;
	updateparticle = NULL;
	isparticlealive = NULL;
	animated = false;
};

void CMMParticleSystem::Reset( LPDIRECT3DDEVICE9 DEVICE,
							   int PARTICLENUM, float PARTICLESIZE,
							   LPInitParticle INITPARTICLE,
							   LPUpdateParticle UPDATEPARTICLE,
							   LPIsParticleAlive ISPARTICLEALIVE,
							   CMMTexture* TEXTURE, bool ANIMATED )
{
	CleanUp();
	particlenum = PARTICLENUM;
	particlesize = PARTICLESIZE;
	initparticle = INITPARTICLE;
	updateparticle = UPDATEPARTICLE;
	isparticlealive = ISPARTICLEALIVE;
	particles = new CMMParticle*[particlenum];
	buffer = new PARTICLEVERTEX[particlenum * 4];
	DEVICE->CreateVertexBuffer( sizeof(PARTICLEVERTEX) * particlenum * 4,
		D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC, D3DFVF_PARTICLEVERTEX, D3DPOOL_DEFAULT,
		&vertex, NULL );
	for( int i = 0; i < particlenum; ++i )
	{
		particles[i] = new CMMParticle;
		initparticle( this, i, true );
		SetVertexData( i );
	};
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::Update( int UPDATES )
{
	for( int i = 0; i < particlenum; ++i )
	{
		updateparticle( this, i );
		if( !isparticlealive( this, i ) ) initparticle( this, i, false );
		SetVertexData( i );
	};
};

CMMParticle* CMMParticleSystem::GetParticle( int PARTICLEINDEX )
{
	if( PARTICLEINDEX >=0 && PARTICLEINDEX < particlenum) return particles[PARTICLEINDEX];
	else return NULL;
};

void CMMParticleSystem::SetTexture( CMMTexture* TEXTURE, bool ANIMATED )
{
	SAFE_RELEASE( texture );
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::SetVertexData( int INDEX )
{
	int i = INDEX * 4;
	buffer[i].position.x = -( particlesize / 2.0f );
	buffer[i].position.y = +( particlesize / 2.0f );
	buffer[i].position.z = 0.0f;
	buffer[i].tex1.x = 0.0f;
	buffer[i].tex1.y = 0.0f;

	buffer[i+1].position.x = -( particlesize / 2.0f );
	buffer[i+1].position.y = -( particlesize / 2.0f );
	buffer[i+1].position.z = 0.0f;
	buffer[i+1].tex1.x = 0.0f;
	buffer[i+1].tex1.y = 1.0f;

	buffer[i+2].position.x = +( particlesize / 2.0f );
	buffer[i+2].position.y = +( particlesize / 2.0f );
	buffer[i+2].position.z = 0.0f;
	buffer[i+2].tex1.x = 1.0f;
	buffer[i+2].tex1.y = 0.0f;

	buffer[i+3].position.x = +( particlesize / 2.0f );
	buffer[i+3].position.y = -( particlesize / 2.0f );
	buffer[i+3].position.z = 0.0f;
	buffer[i+3].tex1.x = 1.0f;
	buffer[i+3].tex1.y = 1.0f;

	buffer[i].color = buffer[i+1].color = buffer[i+2].color = buffer[i+3].color =
		particles[INDEX]->GetColor();
};

void CMMParticleSystem::Draw( LPDIRECT3DDEVICE9 DEVICE, D3DXVECTOR3 EYEPT, D3DXVECTOR3 LOOKATPT )
{
/*	D3DXMATRIXA16 billboardview;
	D3DXVECTOR3 vdir;
	D3DXVec3Subtract( &vdir, &LOOKATPT, &EYEPT );
	if( vdir.x > 0.0)
		D3DXMatrixRotationY( &billboardview, -atan2f( vdir.z, vdir.x ) + D3DX_PI / 2.0f );
	else
		D3DXMatrixRotationY( &billboardview, -atan2f( vdir.z, vdir.x ) - D3DX_PI / 2.0f );
*/
	void* vlock = NULL;
	vertex->Lock( 0, sizeof(buffer), (void**)&vlock, 0 );
	memcpy( vlock, buffer, sizeof(buffer) );
	vertex->Unlock();

	DEVICE->SetFVF( D3DFVF_PARTICLEVERTEX );
	DEVICE->SetStreamSource( 0, vertex, 0, sizeof(PARTICLEVERTEX) );
	DEVICE->SetTexture(0, texture->GetTexture() );
	for(int i = 0; i < particlenum; ++i )
	{
/*		billboardview._41 = particles[i]->GetPosition().x;
		billboardview._42 = particles[i]->GetPosition().y;
		billboardview._43 = particles[i]->GetPosition().z;
		DEVICE->SetTransform( D3DTS_WORLD, &billboardview );
*/ 		DEVICE->DrawPrimitive( D3DPT_TRIANGLESTRIP, i * 4, 2 );
	};

};